#!/bin/bash
#
#---------------------------------------------------------
# A function creating and setting 
# dynamically variables from CMOR Tables for cordex. 
#
# Pure bash tested  with:
#
# GNU bash, version 3.2.25(1)-release 
# GNU bash, version 4.1.2(1)-release
#
# mkolax 20120528
#--------------------------------------------------------

set -e

import_cmor_table(){
while read line; do
if [[ $line =~ (.*)(!.*) ]];then
line=${BASH_REMATCH[1]}
fi
local va_name
if [[ $line =~ ([A-Za-z0-9_]*:)[[:space:]]*(.*) ]];then
    local t_name=${va_name}${BASH_REMATCH[1]//:}
    local t_entry=${BASH_REMATCH[2]}
    if [[ $t_name =~ variable_entry|axis_entry ]];then
        va_name="${t_entry}_"
    continue
    else 
    eval ${1}_${t_name}='${t_entry}'
    continue
    fi
fi
done < ${TABLE_PATH}/${1}
}



# -------------------------------------------------------
#
#                     CORDEX writer 
#
# -------------------------------------------------------

cordex_writer(){

file_in=$1
var_in=$2
run_in=$3
path_out=$4

# --- meta info ---
 ref_time='1949-12-01,00:00'                                               
 cdo='cdo -s'
 freq=day

# --- read simulation info ---
 file_run=$path_meta'run_info_cordex.sh'
 . $file_run

# --- read variable info ---
 file_var=$path_meta'var_info_cordex.sh'
 . $file_var

# --- import cmor tables ---
  import_cmor_table CORDEX_day

# --- variable attributes (CORDEX-CMOR tables) ---
  unset st_name st_name units cell_methods positive

  st_name='CORDEX_day_'$var_in'_standard_name'
  long_name='CORDEX_day_'$var_in'_long_name'
  units='CORDEX_day_'$var_in'_units'
  cell_methods='CORDEX_day_'$var_in'_cell_methods'
  positive='CORDEX_day_'$var_in'_positive'
  

# --- post-processing info ---
 echo
 echo ... VARIABLE NETCDF      ' '... $var_in '|' ${!st_name} '|' ${!long_name} '|' ${!units}
 echo ... SIMULATION    '      '... $run_in '|' $domain '|' $experiment_id '|' $gcm_name '|' $gcm_version_id '|' $rcm_name '|' $rcm_version_id '|'  
 echo ... CALENDAR   '       ' ... $calendar
 echo ... OUTPUT PATH  '    ' ... $path_out
 echo

# -------------------------------------
# ---  DATES in FILE NAME: YYYYMMDD ---
# -------------------------------------
 num_time=`$cdo ntime $file_in`
 f_date=`$cdo showdate -seltimestep,1 $file_in`; f_date=${f_date// /}; f_date=${f_date//-/}
 f_time=`$cdo showtime -seltimestep,1 $file_in`; f_time=${f_time:1:2}
 l_date=`$cdo showdate -seltimestep,$num_time $file_in`; l_date=${l_date// /}; l_date=${l_date//-/}
 l_time=`$cdo showtime -seltimestep,$num_time $file_in`; l_time=${l_time:1:2}

# ---- output file name ---
  file_out=$path_out$var_in'_'$domain'_'$gcm_name'_'$experiment_id'_'$gcm_version_id'_'$rcm_name'_'$rcm_version_id'_'$freq'_'$f_date'-'$l_date'.nc'

# --- remove output file if exists --- 
  [ -f $file_out ] && rm $file_out   


# ---------------------------------------
# --- CONVERT to netcdf and fix TIME  ---
# ---------------------------------------
  echo ... processing ... $file_in
  echo
  $cdo -f nc setmissval,1.e20 -setreftime,$ref_time -settunits,days -settime,12:00:00 -setcalendar,$calendar $file_in $file_out


# -----------------------------------
# ---  2D LONGITUDE and LATITUDE  ---
# -----------------------------------
  file_lonlat=$path_meta'add_lonlat_cordex.sh'
  . $file_lonlat


# -----------------------
# --- TIME ATRIBUTES  ---
# -----------------------
  ncatted -a standard_name,time,o,c,"time" \
          -a long_name,time,o,c,"time" \
          -a bounds,time,o,c,"time_bnds" \
          -a axis,time,o,c,"T" -h $file_out

# ---------------------
# ---  TIME BOUNDS  ---
# ---------------------
 ncap2 -h -O -s 'defdim("bnds",2); time_bnds[$time, bnds]=0.; time_bnds(:, 0)=time-0.5; time_bnds(:, 1)=time+0.5;' $file_out $file_out 


# ---------------------------
# --- VARIABLE ATTRIBUTES ---
# ---------------------------
  ncrename -v $old_name,$var_in -h $file_out
  ncatted -a table,$var_in,d,, \
          -a standard_name,$var_in,o,c,"${!st_name}" \
          -a long_name,$var_in,o,c,"${!long_name}" \
          -a units,$var_in,o,c,"${!units}" \
          -a coordinates,$var_in,o,c,"lon lat" \
          -a missing_value,$var_in,o,f,1.e+20 \
          -a cell_methods,$var_in,o,c,"${!cell_methods}" -h $file_out

[[ ! -z ${!positive} ]] && ncatted -a positive,$var_in,o,c,"${!positive}" -h $file_out 


# -----------------------------
# ---  VERTICAL COORDINATE  ---
# -----------------------------
 case $var_in in
  tas | tasmax | tasmin | huss )  
    height=2;;
  sfcWind | sfcWindmax | wsgsmax | uas | vas)
    height=10;;
 esac


 case $var_in in
      pr | rsds)   
         ncwa -O -a height -h $file_out $file_out
         ncks -O -x -v height -h $file_out $file_out;;

           psl)   
         ncwa -O -a alt -h $file_out $file_out
         ncks -O -x -v alt -h $file_out $file_out;;

    tas | tasmax | tasmin | huss | sfcWind | sfcWindmax | wsgsmax | uas | vas)
         ncap2 -h -O -s 'height(0)='$height'' $file_out $file_out
         ncatted -a long_name,height,o,c,"height above the surface" \
                 -a standard_name,height,o,c,"height" \
                 -a units,height,o,c,"m" \
                 -a positive,height,o,c,"up" -h $file_out
         ncwa -O -a height -h $file_out $file_out;;

  ua[0-9][0-9][0-9] | va[0-9][0-9][0-9] | zg[0-9][0-9][0-9] | hus[0-9][0-9][0-9] | ta[0-9][0-9][0-9] | cll | clm | clh)
         case $var_in in
          cll | clm |clh)
              ncrename -d height,plev -v height,plev -h $file_out
              ncatted -a bounds,plev,o,c,"plev_bnds" -h $file_out;; 
           *) 
              ncrename -d lev,plev -v lev,plev -h $file_out;;
         esac

         ncatted -a standard_name,plev,o,c,"air_pressure" \
                 -a long_name,plev,o,c,"pressure" \
                 -a units,plev,o,c,"Pa" \
                 -a positive,plev,o,c,"down" \
                 -a axis,plev,o,c,"Z" -h $file_out
         ncwa -O -a plev -h $file_out $file_out;;
 esac


# ----------------------------------------
# ---  PLEV_BNDS for CLOUDS in LAYERS  ---
# ----------------------------------------
case $var_in in
   cll) plev_down=100000
        plev_up=68000;;
   clm) plev_down=68000
        plev_up=44000;;
   clh) plev_down=44000
        plev_up=0;;
esac

case $var_in in
       cll | clm | clh) 
             ncap2 -h -O -s 'plev_bnds[$bnds]=0.; plev_bnds(0)='$plev_down'; plev_bnds(1)='$plev_up'; plev=('$plev_down'+'$plev_up')/2.' $file_out $file_out  #'
             ncatted -a long_name,plev_bnds,o,c,"Boundaries of the layer in which cloud area fraction is computed" \
                     -a units,plev_bnds,o,c,"Pa" -h $file_out;;
esac


# -------------------------
# --- GLOBAL ATRIBUTES  ---
# -------------------------
 ncatted -a Conventions,global,o,c,"CF-1.4" \
         -a contact,global,o,c,"$contact" \
         -a creation_date,global,o,c,"$( date +%Y'-'%m'-'%d'-T'%T'Z')" \
         -a experiment,global,o,c,"$experiment" \
         -a experiment_id,global,o,c,"$experiment_id" \
         -a driving_experiment,global,o,c,"$driving_experiment" \
         -a driving_model_id,global,o,c,"$gcm_name" \
         -a driving_model_ensemble_member,global,o,c,"$gcm_version_id" \
         -a driving_experiment_name,global,o,c,"$driving_experiment_name" \
         -a frequency,global,o,c,"$freq" \
         -a institution,global,o,c,"$institution" \
         -a institute_id,global,o,c,"$institute_id" \
         -a model_id,global,o,c,"$rcm_name" \
         -a rcm_version_id,global,o,c,"$rcm_version_id" \
         -a project_id,global,o,c,"$project_id" \
         -a CORDEX_domain,global,o,c,"$domain" \
         -a product,global,o,c,"output" \
         -a references,global,o,c,"$references" \
         -a rossby_comment,global,o,c,"$comment" \
         -a rossby_run_id,global,o,c,$run_in -h $file_out

# --- traking_id attribute (recommended) ---
#ncatted  -a tracking_id,global,o,c,`uuid -v 4` -h $file_out


# --------------------------------
# --- REMOVE GLOBAL ATTRIBUTES ---
# --------------------------------
  ncatted -a history,global,d,, \
          -a CDI,global,d,, \
          -a CDO,global,d,, \
          -a NCO,global,d,, -h $file_out

[ -f $file_lonlat ] && rm $file_lonlat # remove file with 2D coordinates

}

